@@ -323,7 +323,7 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen,
 
 	lprintf ("open_plugin called\n");
 
-	this = (file_driver_t *) xine_xmalloc (sizeof (file_driver_t));
+	this = calloc(1, sizeof (file_driver_t));
 	if (!this)
 		return NULL;
 
@@ -375,7 +375,7 @@ static void *init_class (xine_t *xine, void *data) {
 
 	lprintf ("init class\n");
 
-	this = (file_class_t *) xine_xmalloc (sizeof (file_class_t));
+	this = calloc(1, sizeof (file_class_t));
 	if (!this)
 		return NULL;
 
