@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: dxr3_mpeg_encoders.c,v 1.8 2002/07/17 14:58:12 mroi Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
  
 /* mpeg encoders for the dxr3 video out plugin.
@@ -29,7 +27,6 @@
 #endif
 
 #ifdef HAVE_LIBRTE
-#  define _GNU_SOURCE
 #  include <unistd.h>
 #  include <rte.h>
 #endif
@@ -41,15 +38,19 @@
 #include <stdlib.h>
 #include <sys/ioctl.h>
 #include <string.h>
+#include <dlfcn.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <math.h>
+#include <unistd.h>
+
+#define LOG_MODULE "dxr3_mpeg_encoder"
+/* #define LOG_VERBOSE */
+/* #define LOG */
 
 #include "xineutils.h"
 #include "video_out_dxr3.h"
 
-#define LOG_ENC 1
-
 /* buffer size for encoded mpeg1 stream; will hold one intra frame 
  * at 640x480 typical sizes are <50 kB. 512 kB should be plenty */
 #define DEFAULT_BUFFER_SIZE 512*1024
@@ -106,6 +107,12 @@ static int fame_prepare_frame(fame_data_t *this, dxr3_driver_t *drv,
                               dxr3_frame_t *frame);
 #endif
 
+/* initialization function */
+int        dxr3_lavc_init(dxr3_driver_t *drv, plugin_node_t *node);
+
+/* close function from encoder api */
+static int lavc_on_close(dxr3_driver_t *drv);
+
 
 #ifdef HAVE_LIBRTE
 int dxr3_rte_init(dxr3_driver_t *drv)
@@ -113,13 +120,12 @@ int dxr3_rte_init(dxr3_driver_t *drv)
   rte_data_t* this;
   
   if (!rte_init()) {
-    printf("dxr3_mpeg_encoder: failed to init librte\n");
+    xprintf(drv->class->xine, XINE_VERBOSITY_LOG, _("dxr3_mpeg_encoder: failed to init librte\n"));
     return 0;
   }
   
-  this = malloc(sizeof(rte_data_t));
+  this = calloc(1, sizeof(rte_data_t));
   if (!this) return 0;
-  memset(this, 0, sizeof(rte_data_t));
   
   this->encoder_data.type             = ENC_RTE;
   this->encoder_data.on_update_format = rte_on_update_format;
@@ -141,16 +147,15 @@ static int rte_on_update_format(dxr3_driver_t *drv, dxr3_frame_t *frame)
   double fps;
 
   if (this->context) { /* already running */
-#if LOG_ENC
-    printf("dxr3_mpeg_encoder: closing current encoding context.\n");
-#endif
+    lprintf("closing current encoding context.\n");
     rte_stop(this->context);
     rte_context_destroy(this->context);
     this->context = 0;
   }
   
   if ((frame->vo_frame.pitches[0] % 16 != 0) || (frame->oheight % 16 != 0)) {
-    printf("dxr3_mpeg_encoder: rte only handles video dimensions which are multiples of 16\n");
+    xprintf(drv->class->xine, XINE_VERBOSITY_LOG, 
+	    _("dxr3_mpeg_encoder: rte only handles video dimensions which are multiples of 16\n"));
     return 0;
   }
   
@@ -160,7 +165,7 @@ static int rte_on_update_format(dxr3_driver_t *drv, dxr3_frame_t *frame)
   /* create new rte context */
   this->context = rte_context_new(this->width, this->height, "mp1e", drv);
   if (!this->context) {
-    printf("dxr3_mpeg_encoder: failed to get rte context.\n");
+    xprintf(drv->class->xine, XINE_VERBOSITY_LOG, _("dxr3_mpeg_encoder: failed to get rte context.\n"));
     return 0;
   }
   context = this->context; /* shortcut */
@@ -171,17 +176,17 @@ static int rte_on_update_format(dxr3_driver_t *drv, dxr3_frame_t *frame)
   /* get mpeg codec handle */
   codec = rte_codec_set(context, RTE_STREAM_VIDEO, 0, "mpeg1_video");
   if (!codec) {
-    printf("dxr3_mpeg_encoder: could not create codec.\n");
+    xprintf(drv->class->xine, XINE_VERBOSITY_LOG, _("dxr3_mpeg_encoder: could not create codec.\n"));
     rte_context_destroy(context);
     this->context = 0;
     return 0;
   }
   
-  this->rte_bitrate = drv->config->register_range(drv->config,
-    "dxr3.rte_bitrate", 10000, 1000, 20000,
-    _("Dxr3enc: rte mpeg output bitrate (kbit/s)"), 
-    _("The bitrate the mpeg encoder library librte should use for dxr3's encoding mode"),
-    NULL, NULL);
+  this->rte_bitrate = drv->class->xine->config->register_range(drv->class->xine->config,
+    "dxr3.encoding.rte_bitrate", 10000, 1000, 20000,
+    _("rte mpeg output bitrate (kbit/s)"), 
+    _("The bitrate the mpeg encoder library librte should use for DXR3's encoding mode. "
+      "Higher values will increase quality and CPU usage."), 10, NULL, NULL);
   this->rte_bitrate *= 1000; /* config in kbit/s, rte wants bit/s */
   
   /* FIXME: this needs to be replaced with a codec option call. 
@@ -191,7 +196,7 @@ static int rte_on_update_format(dxr3_driver_t *drv, dxr3_frame_t *frame)
    * and context->video_bytes (= width * height * bytes/pixel)
    */
   rte_set_video_parameters(context, 
-    (frame->vo_frame.format == IMGFMT_YV12 ? RTE_YUV420 : RTE_YUYV),
+    (frame->vo_frame.format == XINE_IMGFMT_YV12 ? RTE_YUV420 : RTE_YUYV),
     context->width, context->height, 
     context->video_rate, context->output_video_bits,
     context->gop_sequence);
@@ -203,48 +208,47 @@ static int rte_on_update_format(dxr3_driver_t *drv, dxr3_frame_t *frame)
    */
   fps = 90000.0 / frame->vo_frame.duration;
   if (!rte_option_set(codec, "virtual_frame_rate", fps))
-    printf("dxr3_mpeg_encoder: WARNING: rte_option_set failed; virtual_frame_rate = %g.\n", fps);
+    xprintf(drv->class->xine, XINE_VERBOSITY_DEBUG, 
+	    "dxr3_mpeg_encoder: WARNING: rte_option_set failed; virtual_frame_rate = %g.\n", fps);
   if (!rte_option_set(codec, "coded_frame_rate", fps))
-    printf("dxr3_mpeg_encoder: WARNING: rte_option_set failed; coded_frame_rate = %g.\n", fps);
+    xprintf(drv->class->xine, XINE_VERBOSITY_DEBUG, 
+	    "dxr3_mpeg_encoder: WARNING: rte_option_set failed; coded_frame_rate = %g.\n", fps);
   if (!rte_option_set(codec, "bit_rate", (int)this->rte_bitrate))
-    printf("dxr3_mpeg_encoder: WARNING: rte_option_set failed; bit_rate = %d.\n", (int)this->rte_bitrate);
+    xprintf(drv->class->xine, XINE_VERBOSITY_DEBUG,
+	    "dxr3_mpeg_encoder: WARNING: rte_option_set failed; bit_rate = %d.\n", (int)this->rte_bitrate);
   if (!rte_option_set(codec, "gop_sequence", "I"))
-    printf("dxr3_mpeg_encoder: WARNING: rte_option_set failed; gop_sequence = \"I\".\n");
+    xprintf(drv->class->xine, XINE_VERBOSITY_DEBUG, 
+	    "dxr3_mpeg_encoder: WARNING: rte_option_set failed; gop_sequence = \"I\".\n");
   /* just to be sure, disable motion comp (not needed in I frames) */
   if (!rte_option_set(codec, "motion_compensation", 0))
-    printf("dxr3_mpeg_encoder: WARNING: rte_option_set failed; motion_compensation = 0.\n");
+    xprintf(drv->class->xine, XINE_VERBOSITY_DEBUG,
+	    "dxr3_mpeg_encoder: WARNING: rte_option_set failed; motion_compensation = 0.\n");
   
   rte_set_input(context, RTE_VIDEO, RTE_PUSH, FALSE, NULL, NULL, NULL);
   rte_set_output(context, mp1e_callback, NULL, NULL);
   
   if (!rte_init_context(context)) {
-    printf("dxr3_mpeg_encoder: cannot init the context: %s\n", context->error);
+    xprintf(drv->class->xine, XINE_VERBOSITY_LOG, 
+	    _("dxr3_mpeg_encoder: cannot init the context: %s\n"), context->error);
     rte_context_destroy(context);
     this->context = 0;
     return 0;
   }
   /* do the sync'ing and start encoding */
   if (!rte_start_encoding(context)) {
-    printf("dxr3_mpeg_encoder: cannot start encoding: %s\n", context->error);
+    xprintf(drv->class->xine, XINE_VERBOSITY_LOG, 
+	    _("dxr3_mpeg_encoder: cannot start encoding: %s\n"), context->error);
     rte_context_destroy(context);
     this->context = 0;
     return 0;
   }
   this->rte_ptr = rte_push_video_data(context, NULL, 0);
   if (!this->rte_ptr) {
-    printf("dxr3_mpeg_encoder: failed to get encoder buffer pointer.\n");
+    xprintf(drv->class->xine, XINE_VERBOSITY_DEBUG, 
+	    "dxr3_mpeg_encoder: failed to get encoder buffer pointer.\n");
     return 0;
   }
   
-  /* set the dxr3 playmode */
-  if (drv->enhanced_mode) {
-    em8300_register_t regs; 
-    regs.microcode_register = 1;
-    regs.reg = 0;
-    regs.val = MVCOMMAND_SYNC;
-    ioctl(drv->fd_control, EM8300_IOCTL_WRITEREG, &regs);
-  }
-  
   return 1;
 }
 
@@ -256,14 +260,14 @@ static int rte_on_display_frame(dxr3_driver_t *drv, dxr3_frame_t *frame)
   if ((this->width == frame->vo_frame.pitches[0]) && (this->height == frame->oheight)) {
     /* This frame belongs to current context. */
     size = frame->vo_frame.pitches[0] * frame->oheight;
-    if (frame->vo_frame.format == IMGFMT_YV12)
+    if (frame->vo_frame.format == XINE_IMGFMT_YV12)
       xine_fast_memcpy(this->rte_ptr, frame->real_base[0], size * 3/2);
     else
       xine_fast_memcpy(this->rte_ptr, frame->real_base[0], size * 2);
     this->rte_ptr = rte_push_video_data(this->context, this->rte_ptr,
       frame->vo_frame.vpts / 90000.0);
   }
-  frame->vo_frame.displayed(&frame->vo_frame);
+  frame->vo_frame.free(&frame->vo_frame);
   return 1;
 }
 
@@ -294,20 +298,15 @@ static void mp1e_callback(rte_context *context, void *data, ssize_t size, void *
   char tmpstr[128];
   ssize_t written;
   
-  if (drv->fd_video == CLOSED_FOR_ENCODER) {
-    snprintf(tmpstr, sizeof(tmpstr), "%s_mv%s", drv->devname, drv->devnum);
-    drv->fd_video = open(tmpstr, O_WRONLY | O_NONBLOCK);
-  }
-  if (drv->fd_video < 0) return;
   written = write(drv->fd_video, data, size);
   if (written < 0) {
-    printf("dxr3_mpeg_encoder: video device write failed (%s)\n",
-      strerror(errno));
+    xprintf(drv->class->xine, XINE_VERBOSITY_DEBUG, 
+	    "dxr3_mpeg_encoder: video device write failed (%s)\n", strerror(errno));
     return;
   }
   if (written != size)
-    printf("dxr3_mpeg_encoder: Could only write %d of %d mpeg bytes.\n",
-      written, size);
+    xprintf(drv->class->xine, XINE_VERBOSITY_DEBUG, 
+	    "dxr3_mpeg_encoder: Could only write %d of %d mpeg bytes.\n", written, size);
 }
 #endif
 
@@ -317,9 +316,8 @@ int dxr3_fame_init(dxr3_driver_t *drv)
 {
   fame_data_t *this;
   
-  this = malloc(sizeof(fame_data_t));
+  this = calloc(1, sizeof(fame_data_t));
   if (!this) return 0;
-  memset(this, 0, sizeof(fame_data_t));
   
   this->encoder_data.type             = ENC_FAME;
   this->encoder_data.on_update_format = fame_on_update_format;
@@ -345,8 +343,8 @@ static int fame_on_update_format(dxr3_driver_t *drv, dxr3_frame_t *frame)
   
   /* if YUY2 and dimensions changed, we need to re-allocate the
    * internal YV12 buffer */
-  if (frame->vo_frame.format == IMGFMT_YUY2) {
-    int image_size = drv->video_width * drv->video_oheight;
+  if (frame->vo_frame.format == XINE_IMGFMT_YUY2) {
+    int image_size = frame->vo_frame.width * frame->oheight;
 
     this->out[0] = xine_xmalloc_aligned(16, image_size * 3/2, 
       (void *)&this->buf);
@@ -357,47 +355,44 @@ static int fame_on_update_format(dxr3_driver_t *drv, dxr3_frame_t *frame)
     memset(this->out[0], 16, image_size);
     memset(this->out[1], 128, image_size/4);
     memset(this->out[2], 128, image_size/4);
-#if LOG_ENC
-    printf("dxr3_mpeg_encoder: Using YUY2->YV12 conversion\n");  
-#endif
+    lprintf("Using YUY2->YV12 conversion\n");  
   }
 
   if (this->context) {
-#if LOG_ENC
-    printf("dxr3_mpeg_encoder: closing current encoding context.\n");
-#endif
+    lprintf("closing current encoding context.\n");
     fame_close(this->context);
     this->context = 0;
   }
   
   this->context = fame_open();
   if (!this->context) {
-    printf("dxr3_mpeg_encoder: Couldn't start the FAME library\n");
+    xprintf(drv->class->xine, XINE_VERBOSITY_LOG, 
+	    _("dxr3_mpeg_encoder: Couldn't start the FAME library\n"));
     return 0;
   }
   
   if (!this->buffer)
     this->buffer = (unsigned char *)malloc(DEFAULT_BUFFER_SIZE);
   if (!this->buffer) {
-    printf("dxr3_mpeg_encoder: Couldn't allocate temp buffer for mpeg data\n");
+    xprintf(drv->class->xine, XINE_VERBOSITY_DEBUG, 
+	    "dxr3_mpeg_encoder: Couldn't allocate temp buffer for mpeg data\n");
     return 0;
   }
 
   this->fp = init_fp;
-  this->fp.quality = drv->config->register_range(drv->config,
-    "dxr3.fame_quality", 90, 10, 100,
-    _("Dxr3enc: fame mpeg encoding quality"),
-    _("The encoding quality of the libfame mpeg encoder library."),
-    NULL,NULL);
-#if LOG_ENC
+  this->fp.quality = drv->class->xine->config->register_range(drv->class->xine->config,
+    "dxr3.encoding.fame_quality", 90, 10, 100,
+    _("fame mpeg encoding quality"),
+    _("The encoding quality of the libfame mpeg encoder library. "
+      "Lower is faster but gives noticeable artifacts. Higher is better but slower."),
+    10, NULL,NULL);
   /* the really interesting bit is the quantizer scale. The formula
    * below is copied from libfame's sources (could be changed in the
    * future) */
-  printf("dxr3_mpeg_encoder: quality %d -> quant scale = %d\n", this->fp.quality,
+  lprintf("quality %d -> quant scale = %d\n", this->fp.quality,
     1 + (30 * (100 - this->fp.quality) + 50) / 100);
-#endif
-  this->fp.width   = drv->video_width;
-  this->fp.height  = drv->video_oheight;
+  this->fp.width   = frame->vo_frame.width;
+  this->fp.height  = frame->oheight;
   this->fp.profile = "mpeg1";
   this->fp.coding  = "I";
 #if LOG_ENC
@@ -408,54 +403,26 @@ static int fame_on_update_format(dxr3_driver_t *drv, dxr3_frame_t *frame)
 
   /* start guessing the framerate */
   fps = 90000.0 / frame->vo_frame.duration;
-  if (fabs(fps - 25) < 0.01) { /* PAL */
-#if LOG_ENC
-    printf("dxr3_mpeg_encoder: setting mpeg output framerate to PAL (25 Hz)\n");
-#endif
-    this->fp.frame_rate_num = 25;
-    this->fp.frame_rate_den = 1; 
-  }  
-  else if (fabs(fps - 24) < 0.01) { /* FILM */
-#if LOG_ENC
-    printf("dxr3_mpeg_encoder: setting mpeg output framerate to FILM (24 Hz)\n");
-#endif
-    this->fp.frame_rate_num = 24;
-    this->fp.frame_rate_den = 1; 
-  }
-  else if (fabs(fps - 23.976) < 0.01) { /* NTSC-FILM */
-#if LOG_ENC
-    printf("dxr3_mpeg_encoder: setting mpeg output framerate to NTSC-FILM (23.976 Hz)\n");
-#endif
+  if (fps < 23.988) { /* NTSC-FILM */
+    lprintf("setting mpeg output framerate to NTSC-FILM (23.976 Hz)\n");
     this->fp.frame_rate_num = 24000;
     this->fp.frame_rate_den = 1001; 
-  }
-  else if (fabs(fps - 29.97) < 0.01) { /* NTSC */
-#if LOG_ENC
-    printf("dxr3_mpeg_encoder: setting mpeg output framerate to NTSC (29.97 Hz)\n");
-#endif
+  } else if (fps < 24.5) { /* FILM */
+    lprintf("setting mpeg output framerate to FILM (24 Hz)\n");
+    this->fp.frame_rate_num = 24;
+    this->fp.frame_rate_den = 1; 
+  } else if (fps < 27.485) { /* PAL */
+    lprintf("setting mpeg output framerate to PAL (25 Hz)\n");
+    this->fp.frame_rate_num = 25;
+    this->fp.frame_rate_den = 1; 
+  } else { /* NTSC */
+    lprintf("setting mpeg output framerate to NTSC (29.97 Hz)\n");
     this->fp.frame_rate_num = 30000;
     this->fp.frame_rate_den = 1001;
   }
-  else { /* try 1/fps, if not legal, libfame will go to PAL */
-    this->fp.frame_rate_num = (int)(fps + 0.5);
-    this->fp.frame_rate_den = 1;
-#if LOG_ENC
-    printf("dxr3_mpeg_encoder: trying to set mpeg output framerate to %d Hz\n",
-      this->fp.frame_rate_num);
-#endif
-  }
   
   fame_init (this->context, &this->fp, this->buffer, DEFAULT_BUFFER_SIZE);
   
-  /* set the dxr3 playmode */
-  if (drv->enhanced_mode) {
-    em8300_register_t regs; 
-    regs.microcode_register = 1;
-    regs.reg = 0;
-    regs.val = MVCOMMAND_SYNC;
-    ioctl(drv->fd_control, EM8300_IOCTL_WRITEREG, &regs);
-  }
-  
   return 1;
 }
 
@@ -468,7 +435,7 @@ static int fame_on_display_frame(dxr3_driver_t *drv, dxr3_frame_t *frame)
 
   if ((frame->vo_frame.width != this->fp.width) || (frame->oheight != this->fp.height)) {
     /* probably an old frame for a previous context. ignore it */
-    frame->vo_frame.displayed(&frame->vo_frame);
+    frame->vo_frame.free(&frame->vo_frame);
     return 1;
   }
 
@@ -481,21 +448,18 @@ static int fame_on_display_frame(dxr3_driver_t *drv, dxr3_frame_t *frame)
   size = fame_encode_frame(this->context, &this->yuv, NULL);
 #endif
 
-  frame->vo_frame.displayed(&frame->vo_frame); 
+  frame->vo_frame.free(&frame->vo_frame); 
   
-  if (drv->fd_video == CLOSED_FOR_ENCODER) {
-    snprintf (tmpstr, sizeof(tmpstr), "%s_mv%s", drv->devname, drv->devnum);
-    drv->fd_video = open(tmpstr, O_WRONLY | O_NONBLOCK);
-  }
-  if (drv->fd_video < 0) return 0;
   written = write(drv->fd_video, this->buffer, size);
   if (written < 0) {
-    printf("dxr3_mpeg_encoder: video device write failed (%s)\n",
+    xprintf(drv->class->xine, XINE_VERBOSITY_DEBUG, 
+	    "dxr3_mpeg_encoder: video device write failed (%s)\n",
       strerror(errno));
     return 0;
   }
   if (written != size)
-    printf("dxr3_mpeg_encoder: Could only write %d of %d mpeg bytes.\n",
+    xprintf(drv->class->xine, XINE_VERBOSITY_DEBUG, 
+	    "dxr3_mpeg_encoder: Could only write %d of %d mpeg bytes.\n",
       written, size);
   return 1;
 }
@@ -527,10 +491,11 @@ static int fame_prepare_frame(fame_data_t *this, dxr3_driver_t *drv, dxr3_frame_
 
   if (frame->vo_frame.bad_frame) return 1;
 
-  if (frame->vo_frame.format == IMGFMT_YUY2) {
+  if (frame->vo_frame.format == XINE_IMGFMT_YUY2) {
     /* need YUY2->YV12 conversion */
     if (!(this->out[0] && this->out[1] && this->out[2]) ) {
-      printf("dxr3_mpeg_encoder: Internal YV12 buffer not created.\n");
+      xprintf(drv->class->xine, XINE_VERBOSITY_DEBUG, 
+	      "dxr3_mpeg_encoder: Internal YV12 buffer not created.\n");
       return 0;
     }
     y = this->out[0] +  frame->vo_frame.width      *  drv->top_bar;
@@ -572,3 +537,33 @@ static int fame_prepare_frame(fame_data_t *this, dxr3_driver_t *drv, dxr3_frame_
   return 1;
 }
 #endif
+
+
+int dxr3_lavc_init(dxr3_driver_t *drv, plugin_node_t *node)
+{
+  void *ffmpeg;
+  int (*init)(dxr3_driver_t *);
+  int result;
+  
+  ffmpeg = dlopen(node->file->filename, RTLD_LAZY);
+  if (!ffmpeg) return 0;
+  
+  init = dlsym(ffmpeg, "dxr3_encoder_init");
+  if (!init) return 0;
+  
+  result = init(drv);
+  /* the close function is implemented here, because it will call dlclose()
+   * and that should not be done be the library we are closing... */
+  drv->enc->on_close = lavc_on_close;
+  drv->enc->handle   = ffmpeg;
+  return result;
+}
+
+static int lavc_on_close(dxr3_driver_t *drv)
+{
+  drv->enc->on_unneeded(drv);
+  dlclose(drv->enc->handle);
+  free(drv->enc);
+  drv->enc = NULL;
+  return 1;
+}
