@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * Color Conversion Utility Functions
  * 
@@ -60,8 +60,6 @@
  * to facilitate these operations (such as the PPC AltiVec pixel-unpacking
  * instructions), these macros will automatically map to those special
  * instructions.
- *
- * $Id: color.c,v 1.29 2006/06/20 00:35:08 dgp85 Exp $
  */
 
 #include "xine_internal.h"
@@ -161,16 +159,14 @@ void (*yuy2_to_yv12)
  * and height passed to it. The width must be divisible by 2.
  */
 void init_yuv_planes(yuv_planes_t *yuv_planes, int width, int height) {
-
-  int plane_size;
+  memset (yuv_planes, 0, sizeof (yuv_planes));
 
   yuv_planes->row_width = width;
   yuv_planes->row_count = height;
-  plane_size = yuv_planes->row_width * yuv_planes->row_count;
 
-  yuv_planes->y = xine_xmalloc(plane_size);
-  yuv_planes->u = xine_xmalloc(plane_size);
-  yuv_planes->v = xine_xmalloc(plane_size);
+  yuv_planes->y = calloc(width, height);
+  yuv_planes->u = calloc(width, height);
+  yuv_planes->v = calloc(width, height);
 }
 
 /*
@@ -179,9 +175,12 @@ void init_yuv_planes(yuv_planes_t *yuv_planes, int width, int height) {
  * This frees the memory used by the YUV planes.
  */
 void free_yuv_planes(yuv_planes_t *yuv_planes) {
-  free(yuv_planes->y);
-  free(yuv_planes->u);
-  free(yuv_planes->v);
+  if (yuv_planes->y)
+    free(yuv_planes->y);
+  if (yuv_planes->u)
+    free(yuv_planes->u);
+  if (yuv_planes->v)
+    free(yuv_planes->v);
 }
 
 /* 
