@@ -393,11 +393,8 @@ const char *xine_get_pluginroot(void) {
   static char pluginroot[1024] = {0, };
 
   if (!pluginroot[0]) {
-    char *sep, *sep2;
-    strcpy (pluginroot, xine_get_plugindir ());
-    sep = strrchr (pluginroot, '/');
-    sep2 = strrchr (pluginroot, '\\');
-    *(sep < sep2 ? sep : sep2) = 0;
+    xine_get_rootdir(pluginroot, sizeof(pluginroot) - strlen(XINE_REL_PLUGINROOT) - 1);
+    strcat(pluginroot, XINE_DIRECTORY_SEPARATOR_STRING XINE_REL_PLUGINROOT);
   }
 
   return pluginroot;
@@ -459,23 +456,22 @@ char *xine_chomp(char *str) {
  * a thread-safe usecond sleep
  */
 void xine_usec_sleep(unsigned usec) {
-#if 0
-#if HAVE_NANOSLEEP
+#ifdef WIN32
+  /* select does not work on win32 */
+  Sleep(usec / 1000);
+#else
+#  if 0
+#    if HAVE_NANOSLEEP
   /* nanosleep is prefered on solaris, because it's mt-safe */
   struct timespec ts, remaining;
-
   ts.tv_sec =   usec / 1000000;
   ts.tv_nsec = (usec % 1000000) * 1000;
   while (nanosleep (&ts, &remaining) == -1 && errno == EINTR)
     ts = remaining;
-#else
-#  if WIN32
-  Sleep(usec / 1000);
-#  else
+#    else
   usleep(usec);
-#  endif
-#endif
-#else
+#    endif
+#  else
   if (usec < 10000) {
       usec = 10000;
   }
@@ -483,6 +479,7 @@ void xine_usec_sleep(unsigned usec) {
   tm.tv_sec  = usec / 1000000;
   tm.tv_usec = usec % 1000000;
   select(0, 0, 0, 0, &tm);
+#  endif
 #endif
 }
 
