@@ -1,27 +1,28 @@
 /*
- *  Copyright (C) 2002-2003 the xine project
+ *  Copyright (C) 2002-2003,2007 the xine project
  *
  *  This file is part of xine, a free video player.
- *  This file is part of gxine, a free video player.
  *
- *  xine is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * The xine-lib XML parser is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
  *
- *  xine is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- *  $Id: xmlparser.c,v 1.13.2.1 2005/02/09 19:39:48 mroi Exp $
+ * The xine-lib XML parser is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
+ * You should have received a copy of the GNU Library General Public
+ * License along with the Gnome Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
+ * Floor, Boston, MA 02110, USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,7 +36,12 @@
 #define LOG
 */
 
+#ifdef XINE_COMPILE
 #include "xineutils.h"
+#else
+#define lprintf(...)
+#define XINE_MALLOC
+#endif
 #include "xmllexer.h"
 #include "xmlparser.h"
 
@@ -72,14 +78,12 @@ static xml_node_t * new_xml_node(void) {
 }
 
 static void free_xml_node(xml_node_t * node) {
-  if (node->name)
-    free (node->name);
-  if (node->data)
-    free (node->data);
+  free (node->name);
+  free (node->data);
   free(node);
 }
 
-static xml_property_t * new_xml_property(void) {
+static xml_property_t *XINE_MALLOC new_xml_property(void) {
   xml_property_t * new_property;
 
   new_property = (xml_property_t*) malloc(sizeof(xml_property_t));
@@ -90,14 +94,12 @@ static xml_property_t * new_xml_property(void) {
 }
 
 static void free_xml_property(xml_property_t * property) {
-  if (property->name)
-    free (property->name);
-  if (property->value)
-    free (property->value);
+  free (property->name);
+  free (property->value);
   free(property);
 }
 
-void xml_parser_init(char * buf, int size, int mode) {
+void xml_parser_init(const char * buf, int size, int mode) {
 
   lexer_init(buf, size);
   xml_parser_mode = mode;
@@ -155,10 +157,15 @@ void xml_parser_free_tree(xml_node_t *current_node) {
 #define STATE_NODE    1
 #define STATE_COMMENT 7
 
-static int xml_parser_get_node (xml_node_t *current_node, char *root_name, int rec) {
-  char tok[TOKEN_SIZE];
-  char property_name[TOKEN_SIZE];
-  char node_name[TOKEN_SIZE];
+static int xml_parser_get_node (xml_node_t *current_node, char *root_name, int rec);
+
+static int _xml_parser_get_node (char ** token_buffer, int * token_buffer_size,
+                                 char ** pname_buffer, int * pname_buffer_size,
+                                 char ** nname_buffer, int * nname_buffer_size,
+                                 xml_node_t *current_node, char *root_name, int rec) {
+  char *tok = *token_buffer;
+  char *property_name = *pname_buffer;
+  char *node_name = *nname_buffer;
   int state = STATE_IDLE;
   int res = 0;
   int parse_res;
@@ -170,7 +177,10 @@ static int xml_parser_get_node (xml_node_t *current_node, char *root_name, int r
 
   if (rec < MAX_RECURSION) {
 
-    while ((bypass_get_token) || (res = lexer_get_token(tok, TOKEN_SIZE)) != T_ERROR) {
+    memset (tok, 0, *token_buffer_size);
+
+    while ((bypass_get_token) || (res = lexer_get_token_d(token_buffer, token_buffer_size, 0)) != T_ERROR) {
+      tok = *token_buffer;
       bypass_get_token = 0;
       lprintf("info: %d - %d : '%s'\n", state, res, tok);
 
@@ -225,6 +235,12 @@ static int xml_parser_get_node (xml_node_t *current_node, char *root_name, int r
 	  if (xml_parser_mode == XML_PARSER_CASE_INSENSITIVE) {
 	    strtoupper(tok);
 	  }
+	  /* make sure the buffer for the node name is big enough */
+	  if (*token_buffer_size > *nname_buffer_size) {
+	    *nname_buffer_size = *token_buffer_size;
+	    *nname_buffer = realloc (*nname_buffer, *nname_buffer_size);
+	    node_name = *nname_buffer;
+	  }
 	  strcpy(node_name, tok);
 	  state = 2;
 	  lprintf("info: current node name \"%s\"\n", node_name);
@@ -291,6 +307,12 @@ static int xml_parser_get_node (xml_node_t *current_node, char *root_name, int r
 	  if (xml_parser_mode == XML_PARSER_CASE_INSENSITIVE) {
 	    strtoupper(tok);
 	  }
+	  /* make sure the buffer for the property name is big enough */
+	  if (*token_buffer_size > *pname_buffer_size) {
+	    *pname_buffer_size = *token_buffer_size;
+	    *pname_buffer = realloc (*pname_buffer, *pname_buffer_size);
+	    property_name = *pname_buffer;
+	  }
 	  strcpy(property_name, tok);
 	  state = 5;
 	  lprintf("info: current property name \"%s\"\n", property_name);
@@ -412,7 +434,7 @@ static int xml_parser_get_node (xml_node_t *current_node, char *root_name, int r
 	}
 	break;
 
-				/* > expected */
+				/* ?> expected */
       case 8:
 	switch (res) {
 	case (T_TI_STOP):
@@ -424,7 +446,7 @@ static int xml_parser_get_node (xml_node_t *current_node, char *root_name, int r
 	}
 	break;
 
-				/* ?> expected */
+				/* > expected */
       case 9:
 	switch (res) {
 	case (T_M_STOP_1):
@@ -452,6 +474,28 @@ static int xml_parser_get_node (xml_node_t *current_node, char *root_name, int r
   }
 }
 
+static int xml_parser_get_node (xml_node_t *current_node, char *root_name, int rec)
+{
+  int res = 0;
+  int token_buffer_size = TOKEN_SIZE;
+  int pname_buffer_size = TOKEN_SIZE;
+  int nname_buffer_size = TOKEN_SIZE;
+  char *token_buffer = calloc(1, token_buffer_size);
+  char *pname_buffer = calloc(1, pname_buffer_size);
+  char *nname_buffer = calloc(1, nname_buffer_size);
+
+  res = _xml_parser_get_node(&token_buffer, &token_buffer_size,
+                             &pname_buffer, &pname_buffer_size,
+                             &nname_buffer, &nname_buffer_size,
+                             current_node, root_name, rec);
+
+  free (token_buffer);
+  free (pname_buffer);
+  free (nname_buffer);
+
+  return res;
+}
+
 int xml_parser_build_tree(xml_node_t **root_node) {
   xml_node_t *tmp_node;
   int res;
@@ -470,7 +514,7 @@ int xml_parser_build_tree(xml_node_t **root_node) {
   return res;
 }
 
-char *xml_parser_get_property (xml_node_t *node, const char *name) {
+const char *xml_parser_get_property (const xml_node_t *node, const char *name) {
 
   xml_property_t *prop;
 
@@ -490,11 +534,11 @@ char *xml_parser_get_property (xml_node_t *node, const char *name) {
   return NULL;
 }
 
-int xml_parser_get_property_int (xml_node_t *node, const char *name, 
+int xml_parser_get_property_int (const xml_node_t *node, const char *name, 
 				 int def_value) {
 
-  char *v;
-  int   ret;
+  const char *v;
+  int         ret;
 
   v = xml_parser_get_property (node, name);
 
@@ -507,20 +551,17 @@ int xml_parser_get_property_int (xml_node_t *node, const char *name,
     return ret;
 }
 
-int xml_parser_get_property_bool (xml_node_t *node, const char *name, 
+int xml_parser_get_property_bool (const xml_node_t *node, const char *name, 
 				  int def_value) {
 
-  char *v;
+  const char *v;
 
   v = xml_parser_get_property (node, name);
 
   if (!v)
     return def_value;
 
-  if (!strcasecmp (v, "true"))
-    return 1;
-  else
-    return 0;
+  return !strcasecmp (v, "true");
 }
 
 static int xml_escape_string_internal (char *buf, const char *s,
@@ -553,34 +594,18 @@ static int xml_escape_string_internal (char *buf, const char *s,
 
 char *xml_escape_string (const char *s, xml_escape_quote_t quote_type)
 {
-  char *buf = xine_xmalloc (xml_escape_string_internal (NULL, s, quote_type));
+  char *buf = calloc (1, xml_escape_string_internal (NULL, s, quote_type));
   return buf ? (xml_escape_string_internal (buf, s, quote_type), buf) : NULL;
 }
 
-static void printf_indent (int indent, const char *format, ...) {
-
-  int     i ;
-  va_list argp;
-
-  for (i=0; i<indent; i++)
-    printf (" ");
-
-  va_start (argp, format);
-    
-  vprintf (format, argp);
-
-  va_end (argp);
-}
-
-static void xml_parser_dump_node (xml_node_t *node, int indent) {
+static void xml_parser_dump_node (const xml_node_t *node, int indent) {
 
   xml_property_t *p;
   xml_node_t     *n;
-  int             l;
 
-  printf_indent (indent, "<%s ", node->name);
+  printf ("%*s<%s ", indent, "", node->name);
 
-  l = strlen (node->name);
+  size_t l = strlen (node->name);
 
   p = node->props;
   while (p) {
@@ -589,8 +614,7 @@ static void xml_parser_dump_node (xml_node_t *node, int indent) {
     free (value);
     p = p->next;
     if (p) {
-      printf ("\n");
-      printf_indent (indent+2+l, "");
+      printf ("\n%*s", indent+2+l, "");
     }
   }
   printf (">\n");
@@ -603,9 +627,9 @@ static void xml_parser_dump_node (xml_node_t *node, int indent) {
     n = n->next;
   }
 
-  printf_indent (indent, "</%s>\n", node->name);
+  printf ("%*s</%s>\n", indent, "", node->name);
 }
 
-void xml_parser_dump_tree (xml_node_t *node) {
+void xml_parser_dump_tree (const xml_node_t *node) {
   xml_parser_dump_node (node, 0);
 }
